package Bot::BasicBot::Pluggable::Module::Notes::Store::DBIC::Schema;

# Created by DBIx::Class::Schema::Loader
# DO NOT MODIFY THE FIRST PART OF THIS FILE

use strict;
use warnings;

use base 'DBIx::Class::Schema';

__PACKAGE__->load_namespaces;


# Created by DBIx::Class::Schema::Loader v0.06001 @ 2010-04-20 21:49:15
# DO NOT MODIFY THIS OR ANYTHING ABOVE! md5sum:HtzJEYGuZaHjnSuN8FvSBA


# You can replace this text with custom content, and it will be preserved on regeneration
1;
